HookOutput class sealed
The output from a hook/build.dart or hook/link.dart.
The shared properties of a BuildOutput and a LinkOutput. This abstraction makes it easier to design APIs intended for both kinds of build hooks, building and linking.
- Implementers
Properties
-
dependencies
→ List<
Uri> -
The files used by this build.
no setter
- hashCode → int
-
The hash code for this object.
no setterinherited
-
json
→ Map<
String, Object?> -
The underlying json configuration of this HookOutput.
no setter
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
- timestamp → DateTime
-
Start time for the build of this output.
no setter
Methods
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
-
toString(
) → String -
A string representation of this object.
override
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited