LinkOutputMaybeFailure class sealed

Either a successful LinkOutput or a LinkOutputFailure.

Implementers

Constructors

LinkOutputMaybeFailure(Map<String, Object?> json)
factory

Properties

hashCode int
The hash code for this object.
no setterinherited
json Map<String, Object?>
The JSON representation of this output.
no setter
runtimeType Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() String
A string representation of this object.
inherited

Operators

operator ==(Object other) bool
The equality operator.
inherited