LinkOutputMetadataBuilder class final

The builder for LinkOutputBuilder.metadata.

Properties

hashCode int
The hash code for this object.
no setterinherited
runtimeType Type
A representation of the runtime type of the object.
no setterinherited

Methods

add(String packageName, String key, Object value) → void
Sets the metadata value for the given key to be sent to packageName.
addAll(String packageName, Map<String, Object> metadata) → void
Adds all entries from metadata to be sent to packageName.
noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() String
A string representation of this object.
inherited

Operators

operator ==(Object other) bool
The equality operator.
inherited