HookInputBuilder class sealed
The builder for HookInput.
- Implementers
Properties
- config → HookConfigBuilder
-
The configuration for this hook input.
no setter
- hashCode → int
-
The hash code for this object.
no setterinherited
-
json
→ Map<
String, Object?> -
The JSON representation of this hook input builder.
no setter
- runtimeType → Type
-
A representation of the runtime type of the object.
no setterinherited
Methods
-
addExtension(
ProtocolExtension extension) → void - Adds the protocol extension to this hook input.
-
computeChecksum(
) → String - Constructs a checksum for a BuildInput.
-
noSuchMethod(
Invocation invocation) → dynamic -
Invoked when a nonexistent method or property is accessed.
inherited
- Sets up the hook input.
-
toString(
) → String -
A string representation of this object.
inherited
Operators
-
operator ==(
Object other) → bool -
The equality operator.
inherited